"use strict";

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const fs = require('fs-extra');

const path = require('path');

const slash = require('slash');

const _require = require('vue-sfc-parser'),
      parseComponent = _require.parseComponent;

const vtc = require('vue-template-compiler');

function addBlock() {
  return '\r\n';
}

function main(options) {
  const insertAttr = options.insertAttr;

  function addAttrs(node) {
    const addtionAttrs = insertAttr(node);
    const attrsMap = node.attrsMap;

    const finalAttrs = _objectSpread({}, attrsMap, addtionAttrs);

    return Object.entries(finalAttrs).map(([k, v]) => {
      return `${k}="${v}"`;
    }).join(" ");
  }

  function addChildren(node) {
    const children = node.children;

    if (!children || !children.length) {
      return [];
    }

    return children.map(child => {
      return traverl(child);
    });
  }

  function traverl(node) {
    let str = '';

    if (node.type === 2) {
      return node.text;
    } else if (node.type === 3) {
      if (node.text.trim()) {
        if (node.isComment) {
          return `<!-- ${node.text} -->`;
        } else {
          return node.text;
        }
      }

      return '';
    }

    function makeOpenTag(tag) {
      return `<${tag} `;
    }

    const start = makeOpenTag(node.tag);
    const attrs = addAttrs(node);

    function closeStartTag() {
      str += ' >' + addBlock();
    }

    const children = addChildren(node);
    str += start + attrs;
    closeStartTag();
    str += children.join(addBlock());
    makeCloseTag(node.tag);

    function makeCloseTag(tag) {
      str += `</${tag}>`;
    }

    return str;
  }

  return traverl;
}

function chuncakeExpression(expression) {
  const reg = /^\"(.+)\"/gi;
  console.log(expression, ' ---');
  let tExpression = reg.exec(expression);

  if (tExpression) {
    tExpression = tExpression[1].split(".");

    if (tExpression.length > 1) {
      return tExpression.pop();
    } else {
      return tExpression.shift();
    }
  }

  return expression;
}

const astTraver = main({
  insertAttr: node => {
    const tag = node.tag,
          attrsMap = node.attrsMap;

    function getName() {
      const expression = node.model.expression,
            parent = node.parent;

      if (parent.tag == 'el-form-item' && parent.attrsMap.prop) {
        return chuncakeExpression(parent.attrsMap.prop);
      }

      return chuncakeExpression(expression);
    }

    function nodeToString(node) {
      const tag = node.tag,
            attrsMap = node.attrsMap;
      let str = '';
      Object.entries(attrsMap).forEach(([k, v]) => {
        str += ` ${k}="${v}" `;
      });
      return `<${tag} ${str} > ...`;
    }

    const formElements = ['el-input', 'el-select', 'el-input-number', 'el-radio-group', 'el-switch', 'el-checkbox-group', 'el-date-picker', 'el-time-picker'];

    if (formElements.find(elName => elName === tag)) {
      return {
        name: getName()
      };
    }

    if (tag === 'el-button' && !!attrsMap.name === false) {
      const children = node.children;

      if (children.find(child => {
        return child.text.indexOf('取消') > -1;
      })) {
        return {
          name: 'btn-cancel'
        };
      }

      let dirty = false;
      [["保存", 'btn-save'], ["取消", "btn-cancel"], ["删除", "btn-delete"], ["清空", "btn-clear"], ["新增", "btn-create"]].forEach(([k, name]) => {
        if (children.find(child => {
          return child.text.indexOf(k) > -1;
        })) {
          dirty = {
            name: name
          };
        }
      });

      if (dirty) {
        return dirty;
      }

      console.error('需要添加 name', nodeToString(node));
    }

    function isRouteSpan(node) {
      const attrsList = node.attrsList;
      return attrsList.some(attr => '$router');
    }

    if (tag === 'span' && isRouteSpan(node)) {
      console.warn('需要添加 name', nodeToString(node));
    } // console.log(node)


    return {};
  }
});
const args = process.argv.slice(2);

function go(filePath) {
  filePath = slash(filePath);
  const isDirectory = fs.lstatSync(filePath).isDirectory();

  if (isDirectory) {
    console.log(filePath, ' --- ');
    fs.readdirSync(filePath).forEach(file => {
      const childFilePath = filePath + '\\' + file;
      go(childFilePath);
    });
    return;
  } else if (!fs.lstatSync(filePath).isFile() || !filePath.endsWith('.vue')) {
    return;
  }

  console.log(`Working: ${filePath}`);
  const fileContent = fs.readFileSync(filePath).toString();

  const _parseComponent = parseComponent(fileContent),
        template = _parseComponent.template,
        script = _parseComponent.script,
        styles = _parseComponent.styles;

  const astObj = vtc.compile(template.content, {
    comments: true
  });
  const repalcedTemplate = astTraver(astObj.ast);
  fs.removeSync(filePath);
  const stylesString = styles.map(singleStyleElement => {
    const attrs = singleStyleElement.attrs ? Object.keys(singleStyleElement.attrs).map(key => {
      return `${key}="${singleStyleElement[key]}"`;
    }).join(' ') : '';
    return `<style ${attrs}>${singleStyleElement.content}</style>`;
  });
  fs.writeFileSync(filePath, [`<template>${repalcedTemplate}</template>`, `<script>${script.content}</script>`].concat(stylesString).join(addBlock()));
}

function run(options) {
  console.log(options);

  if (!options.length) {
    return;
  }

  const projectCwd = process.cwd();
  let dir = options[0];

  if (dir === '--dir') {
    dir = options[1];
    dir = path.join(projectCwd, dir);
    go(dir);
  } else {
    // file
    go(path.join(projectCwd, dir));
  }
}

run(args);